#-------------------------------------------------------------------------------
# cdcatR: An R Package for Cognitive Diagnostic Computerized Adaptive Testing
# Manuscript submitted for publication at {psych}
# Sample code
#-------------------------------------------------------------------------------

library(cdcatR)

#-------------------------------------------------------------------------------
# Figure 3
#-------------------------------------------------------------------------------

Q <- cdcatR::sim180DINA$simQ
dat <- cdcatR::sim180DINA$simdat
#----------Model estimation----------#
fit.GDINA <- GDINA::GDINA(dat = dat, Q = Q, model = "DINA") # GDINA package
fit.GDINA$catprob.parm # item parameters
fit.CDM <- CDM::gdina(data = dat, q.matrix = Q, rule = "DINA") # CDM package
fit.CDM$coef # item parameters
#---------------CD-CAT---------------#
cdcat.GDINA <- cdcatR::cdcat(fit = fit.GDINA)
cdcat.CDM <- cdcatR::cdcat(fit = fit.CDM)
comp <- cdcatR::cdcat.summary(list(cdcat.GDINA, cdcat.CDM), alpha = cdcatR::sim180DINA$simalpha, label = c("GDINA", "CDM"))

#-------------------------------------------------------------------------------
# Figure 4
#-------------------------------------------------------------------------------

Q <- cdcatR::sim180DINA$simQ
dat <- cdcatR::sim180DINA$simdat
resGDINA <- GDINA::GDINA(dat = dat, Q = Q, model = "GDINA", verbose = FALSE)
LR2.GDINA <- cdcatR::LR.2step(fit = resGDINA) 
mean(LR2.GDINA$models.adj.pvalues[which(rowSums(Q) != 1)] == cdcatR::sim180DINA$specifications$item.bank$
       specifications$model[which(rowSums(Q) != 1)])

#-------------------------------------------------------------------------------
# Figure 5
#-------------------------------------------------------------------------------

#----------Item Bank Generation 1: Q is provided----------#
Q <- cdcatR::sim180GDINA$simQ
bank <- cdcatR::gen.itembank(Q = Q, mean.IQ = .60, range.IQ = .20, model = "DINA")
#----------Data Generation: Uniform----------#
N <- 1000
simdata.uni <- cdcatR::gen.data(N, item.bank = bank, seed = 12345)
barplot(table(apply(simdata.uni$simalpha, 1, function(x) paste0(x, collapse = "")))/N)
#----------Data Generation: Multivariate normal----------#
K <- ncol(Q)
cutoffs <- qnorm(c(1:K)/(K+1))
m <- rep(0,K)
vcov <- matrix(0.5,K,K)
diag(vcov) <- 1
simdata.mvn <- cdcatR::gen.data(N, item.bank = bank, att.dist = "mvnorm",
                                mvnorm.parm = list(mean = m, sigma = vcov, cutoffs = cutoffs), seed = 12345)
barplot(table(apply(simdata.mvn$simalpha, 1, function(x) paste0(x, collapse = "")))/N)
#----------Data Generation: User-specified attribute profiles----------#
att.profiles <- matrix(data = c(1,0,0,0,0, 1,1,0,0,0, 1,1,1,0,0, 1,1,1,1,1), ncol = K, byrow = TRUE)
simdata.user <- cdcatR::gen.data(item.bank = bank, att.profiles = 
                                   att.profiles, seed = 12345)
barplot(table(apply(simdata.user$simalpha, 1, function(x) paste0(x, collapse = "")))/N)
#----------Item Bank and Data Generation 2: Q is generated----------#
bank <- cdcatR::gen.itembank(gen.Q = list(J = 200, K = 5, propK.J = c(0.4, 0.3, 0.2, 0.1), nI = 3),
                             mean.IQ = .80, range.IQ = .10, seed = 12345)
table(rowSums(bank$simQ))/nrow(bank$simQ) # should be similar to propK.J
colSums(bank$simQ) # should be greater than minJ.K
parm <- bank$simcatprob.parm
IQ <- unlist(lapply(parm, function(x) {x[length(x)] - x[1]}))
mean(IQ) # should be similar to mean.IQ
range(IQ)[2] - range(IQ)[1] # should be similar to range.IQ
N <- 1000
simdata.uni <- cdcatR::gen.data(N, item.bank = bank, seed = 12345)

#-------------------------------------------------------------------------------
# Appendix A
#-------------------------------------------------------------------------------

library(cdcatR) # v1.0.3
seed <- 507

#--------Data generation and model calibration  --------#
bank <- cdcatR::gen.itembank(gen.Q = list(J = 180, K = 5, propK.J = c(0.4, 0.3, 0.3), nI = 3, minJ.K = 10), mean.IQ = 0.90, range.IQ = 0.20, model = "DINA", seed = seed)
simdat <- cdcatR::gen.data(N = 500, item.bank = bank, seed = seed)
fit <- GDINA::GDINA(simdat$simdat, simdat$simQ, model = "DINA", catprob.parm = simdat$specifications$item.bank$simcatprob.parm, control = list(maxitr = 0))
apply(coef(fit, what = "gs"), 2, summary)

#--------CD-CAT --------#
res.GDI <- cdcatR::cdcat(fit = fit, MAXJ = 15, startRule = "max")
res.GDI.randomstart <- cdcatR::cdcat(fit = fit, MAXJ = 15, startRule = 
                                       "random", NPS.args = list(seed = seed))
res.GDI.progm2 <- cdcatR::cdcat(fit = fit, MAXJ = 15, itemExposurecontrol = "pro-gressive", b = -2, NPS.args = list(seed = seed))
res.GDI.prog2 <- cdcatR::cdcat(fit = fit, MAXJ = 15, itemExposurecontrol = "pro-gressive", b = 2, NPS.args = list(seed = seed))
res.PWKL <- cdcatR::cdcat(fit = fit, MAXJ = 15, itemSelect = "PWKL", NPS.args = list(seed = seed))
res.NPS <- cdcatR::cdcat(fit = fit, dat = simdat$simdat, MAXJ = 15, 
                         itemSelect = "NPS", NPS.args = list(gate = "AND", seed = seed))
res.random <- cdcatR::cdcat(fit = fit, MAXJ = 15, itemSelect = "random", NPS.args = list(seed = seed))
res.GDI.precision <- cdcatR::cdcat(fit = fit, FIXED.LENGTH = FALSE, 
                                   precision.cut = 0.95, NPS.args = list(seed = seed))
res.GDI.precision.prog2 <- cdcatR::cdcat (fit = fit, FIXED.LENGTH = FALSE, 
                                          precision.cut = 0.95, itemExposurecontrol = "progressive", b = 2, NPS.args = list(seed = seed))

#--------Results --------#
comp1 <- cdcatR::cdcat.summary(cdcat.obj = list(res.GDI, res.GDI.randomstart, res.GDI.progm2, res.GDI.prog2, res.PWKL, res.NPS, res.random), alpha = 
                                 simdat$simalpha, label = c("GDI", "GDI.randomstart", "GDI.progm2", "GDI.prog2", "PWKL", "NPS", "random"))

png(filename = "F6.png", width = 600, height = 400, res = 125)
comp1$recovery$PCVcomp + ggplot2::scale_colour_manual(values = c("#558776", "#A3A847","#7F8B52", "#3B5E52", "#DA723C", "#04009A", "#91091E"))
dev.off()
comp1$item.exposure$stats
comp1$item.exposure$plot

comp2 <- cdcatR::cdcat.summary(cdcat.obj = list(res.GDI.precision, res.GDI.precision.prog2), 
                               alpha = simdat$simalpha, label = c("GDI precision", "GDI.prog2 precision"))
comp2$recovery
comp2$item.exposure$stats
comp2$CATlength$stats

png(filename = "F7.png", width = 600, height = 400, res = 125)
comp2$CATlength$plot
dev.off()

png(filename = "F8.png", width = 700, height = 800, res = 125)
comp2$item.exposure$plot
dev.off()

save(list = ls(), file = "psych_illustration.RData") 
